#include <avr/io.h>

#define myclear() { PORTB &= ~( 1 << PB6 ); }
#define myset() { PORTB |= 1 << PB6; }
#define mytoggle() { PORTB ^= 1 << PB6; }

#define mylongwait() { uint16_t c1, c2; c1=TCNT1; do { c2 = TCNT1;} while((c2-c1)<1000);}
#define mywait() { uint16_t c1, c2; c1=TCNT1; do { c2 = TCNT1;} while((c2-c1)<100);}

//#define TEST

void poweron()
{
    mytoggle();
#ifndef TEST
    // clear PD6 (to GND so it power on the ATX PSU)
    PORTD &= ~( 1 << PORTD6 );

    // and set to output
    DDRD |= 1 << DDD6;

    // wait some time
    {
        uint16_t c1, c2;
        c1 = TCNT1;
        do {
            c2 = TCNT1;
        } while( ( c2 - c1 ) < 1562 );
        // almost tenth of a second
    }

    // and change the pin to input again
    DDRD &= ~( 1 << DDD6 );
#endif
}

#define FUZZY_DELTA 2

uint8_t key_values[] = {
                        9, 24, 9, 6, 9, 6, 9, 6,
                        9, 6, 9, 6, 9, 6, 9, 6,
                        9, 6, 9, 24, 9, 24, 9, 24,
                        9, 24, 9, 24, 9, 24, 9, 24,
                        9, 24, 9, 24, 9, 6, 9, 6,
                        9, 6, 9, 6, 9, 6, 9, 6,
                        9, 6, 9, 6, 9, 24, 9, 24,
                        9, 24, 9, 24, 9, 24, 9, 24,
                        9 };

int main( void )
{
    uint8_t v_old, v;
    uint16_t c;
    uint8_t values_pos = 0;
    uint8_t mode = 0;

    DDRB |= 1 << DDB6;
    DDRD = 0;
    myclear();

    // set timer1 counter to freq/256
    TCCR1B = ( TCCR1B & ( ~7 ) ) | ( 1 << 2 );

    // reset prescaler
    GTCCR |= 1;

    v_old = ( PIND ) & ( 1 << PIND2 );
    while ( 1 ) {
        do {
            v = ( PIND ) & ( 1 << PIND2 );
        } while ( v == v_old );
        c = TCNT1;

        // reset counter
        TCNT1 = 0;
        
        switch ( mode ) {
            case 0:
                // no signal is HIGH
                // so this is first falling edge

                if ( ! v ) {
                    mode = 1;
                }
                break;
            case 1:
                // raising edge
                if ( c >= 135 && c < 150 ) {
                    mode = 2;
                } else {
                    mode = 0;
                }
                break;
            case 2:
                // falling edge
                if ( c >= 65 && c < 75 ) {
                    mode = 3;
                    values_pos = 0;
                } else {
                    mode = 0;
                }
                break;
            case 3:
                if ( values_pos < ( sizeof( key_values ) / sizeof( key_values[0] ) ) ) {
                    if ( c >= key_values[values_pos] &&
                         c <= ( key_values[values_pos] + FUZZY_DELTA ) ) {
                        values_pos++;
                    } else {
                        mode = 0;
                    }
                }
                if ( values_pos >= ( sizeof( key_values ) / sizeof( key_values[0] ) ) ) {
                    // key hit

                    poweron();

                    mode = 0;
                }
                break;
            default:
                break;
        }

        v_old = v;
    }

    return 0;
}
